# -*- coding: utf-8 -*-

import csv
from django_importer.importers.base import Importer
from django.utils.encoding import smart_str, force_unicode

def convert_string(s):
    return force_unicode(smart_str(s)).strip()

class CSVImporter(Importer):
    """
    Import models from a local CSV file. Requires `csv` module.
    """
    
    def load(self, source):
        """
        Opens the source file.
        """
        self.source = open(self.source, 'rb')    
        self.loaded = True

    def unload(self):
        """
        Closes the input file to free resources.
        """
        self.source.close()
        self.loaded = False

    def get_items(self):
        """
        Iterator to read the rows of the CSV file.
        """
        # Get the csv reader
        reader = csv.reader(self.source)
        # Get the headers from the first line
        headers = reader.next()
        # Read each line yielding a dictionary mapping
        # the column headers to the row values
        for row in reader:
            # Skip empty rows
            if not row:
                continue
            yield dict(zip(headers, row))
    
    def get_value(self, item, source_name):
        """
        This method receives an item from the source and a source name,
        and returns the text content for the `source_name` node.
        """
        val = item.get(source_name.encode('utf-8'), None)
        if val is not None:
            val = convert_string(val)
        return val




# -*- coding: utf-8 -*-

from django_importer.importers.base import Importer
from django.utils.encoding import smart_str

try:
    from django.utils.encoding import force_text
except ImportError:
    from django.utils.encoding import force_unicode as force_text

# Try to import the best match for `ElementTree`.
# Prioritizes the C port `cElementTree` for *much* better performance.
# Starting from Python 2.5, `cElementTree` is found on `xml.etree`
try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    try:
        import cElementTree as ElementTree
    except ImportError:
        from elementtree import ElementTree

class XMLImporter(Importer):
    """
    Import models from a local XML file. Requires `ElementTree`.
    """

    # The name of the XML node that represents an item
    item_tag_name = 'item'

    def load(self, source):
        """
        Doesn't load anything, just changes the source property,
        `ElementTree` is capable of using a file stream or path,
        so the source type doesn't matter.
        """
        self.source = source
        self.loaded = True

    def get_items(self):
        """
        Iterator of the list of items in the XML source.
        """
        # Use `iterparse`, it's more efficient, specially for big files
        for event, item in ElementTree.iterparse(self.source):
            if item.tag == self.item_tag_name:
                yield item
                # Releases the item from memory
                item.clear()

    def get_value(self, item, source_name):
        """
        This method receives an item from the source and a source name,
        and returns the text content for the `source_name` node.
        """
        return force_text(smart_str(item.findtext(source_name))).strip()




# -*- coding: utf-8 -*-

import StringIO
import gzip
import urllib2

def download_file(url, dest):
    """
    Downloads a HTTP resource from `url` and save to `dest`.
    
    Capable of dealing with Gzip compressed content.
    """
    
    # Create the HTTP request
    request = urllib2.Request(url)
    
    # Add the header to accept gzip encoding
    request.add_header('Accept-encoding', 'gzip')
    
    # Open the request
    opener = urllib2.build_opener()
    response = opener.open(request)
    
    # Retrieve data
    data = response.read()
    
    # If the data is compressed, put the data in a stream and decompress
    if response.headers.get('content-encoding', '') == 'gzip':
        stream = StringIO.StringIO(data)
        gzipper = gzip.GzipFile(fileobj=stream)
        data = gzipper.read()
    
    # Write to a file
    f = open(dest, 'wb')
    f.write(data)
    f.close()



# -*- coding: utf-8 -*-

"""
= django-importer =

Importers for Django models.
Developed and maintained by Enrico Batista da Luz <rico.bl@gmail.com>

To use an existent importer, configure the class properties:

  * `model`: the model to create instances from the data.
  * `fields`: which fields in your model you want to import from the source.
  * `field_map`: the mapping of your field models to the source fields.
  * `unique_fields`: fields that identify an item as unique.

To create your own importer for a different source type, extend `ImporterBase`
and all methods with `NotImplementedError`. Or extend one of the available
importers.
"""

from traceback import format_exception
import sys

class Importer(object):
    """
    Base class to create importers for different sources.
    """
    
    fields = ()
    field_map = {}
    unique_fields = ()
    model = None
    
    def __init__(self, source=None):
        self.source = source
        self.loaded = False
        self.errors = []
    
    def save_error(self, data, exception_info):
        """
        Saves an error in the error list. 
        """
        # TODO: what to do with errors? Let it flow? Write to a log file?
        self.errors.append({'data': data,
                            'exception': ''.join(format_exception(*exception_info)),
                            })
    
    def parse(self):
        """
        Parses all data from the source, saving model instances.
        """
        # Checks if the source is loaded
        if not self.loaded:
            self.load(self.source)
        
        for item in self.get_items():
            # Parse the fields from the source into a dict
            data = self.parse_item(item)
            # Get the instance from the DB, or a new one
            instance = self.get_instance(data)
            # Feed instance with data
            self.feed_instance(data, instance)
            # Try to save the instance or keep the error
            try:
                self.save_item(item, data, instance)
            except Exception as e:
                self.save_error(data, sys.exc_info())

        # Unload the source
        self.unload()
    
    def parse_item(self, item):
        """
        Receives an item and returns a dictionary of field values.
        """
        # Create a dictionary from values for each field
        parsed_data = {}
        
        for field_name in self.fields:
            # A field-name may be mapped to another identifier on the source,
            # it could be a XML path or a CSV column name / position.
            # Defaults to the field-name itself.
            source_name = self.field_map.get(field_name, field_name)
            
            # Uses a custom method "parse_%(field_name)"
            # or get the value from the item
            parse = getattr(self, 'parse_%s' % field_name, None)
            if parse:
                value = parse(item, field_name, source_name)
            else:
                value = self.get_value(item, source_name)
                
            # Add the value to the parsed data
            parsed_data[field_name] = value
        return parsed_data
    
    def get_instance(self, data):
        """
        Get an item from the database or an empty one if not found.
        """
        # Get unique fields
        unique_fields = self.unique_fields
        
        # If there are no unique fields option, all items are new
        if not unique_fields:
            return self.model()
        
        # Build the filter
        filter = dict([(f, data[f]) for f in unique_fields])
        
        # Get the instance from the DB or use a new instance
        try:
            instance = self.model._default_manager.get(**filter)
        except self.model.DoesNotExist:
            return self.model()
        
        return instance
    
    def feed_instance(self, data, instance):
        """
        Feeds a model instance using parsed data (usually from `parse_item`).
        """
        for prop, val in data.items():
            setattr(instance, prop, val)
        return instance
    
    def save_item(self, item, data, instance, commit=True):
        """
        Saves a model instance to the database.
        """
        if commit:
            instance.save()
        return instance
    
    def unload(self):
        """
        Unloads the source file.
        Useful to close open files and free resources.

        Not a required method by subclasses.

        Rembember to unset the ``loaded`` when extending this method.
        """
        self.loaded = False
    
    ### ABSTRACT METHODS #############
    
    def load(self, source):
        """
        Load data from the source.
        
        Must be implemented on subclasses or will raise `NotImplementedError`.
        """
        raise NotImplementedError

    def get_items(self):
        """
        Get the list of items for the current source.
        Must return an iterable.
        
        Must be implemented on subclasses or will raise `NotImplementedError`.
        """
        raise NotImplementedError
    
    def get_value(self, item, source_name):
        """
        Receives an item from the source and a source name,
        and must return the value for the field.
        
        Must be implemented on subclasses or will raise `NotImplementedError`.
        """
        raise NotImplementedError




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# <django-importer - Importers for Django models>
# Copyright (C) <2009>  Enrico Batista da Luz <rico.bl@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django_importer import version
from setuptools import setup

setup(
    name='django-importer',
    version=version,
    description='Data importers for Django models',
    author=u'Enrico Batista da Luz',
    author_email='rico.bl@gmail.com',
    url='http://github.com/ricobl/django-importer/',
    packages=['django_importer', 'django_importer.importers'],
)

